@@ -19,10 +19,12 @@ class IntelOwl:
         instance_url: str,
         certificate: str = None,
         logger: logging.Logger = None,
+        cli: bool = False,
     ):
         self.token = token
         self.instance = instance_url
         self.certificate = certificate
+        self.cli = cli
         if logger:
             self.logger = logger
         else:
@@ -35,12 +37,12 @@ class IntelOwl:
         """
         if not hasattr(self, "_session"):
             session = requests.Session()
-            if self.certificate:
+            if self.certificate is not None:
                 session.verify = self.certificate
             session.headers.update(
                 {
                     "Authorization": f"Token {self.token}",
-                    "User-Agent": "IntelOwlClient/3.0.0",
+                    "User-Agent": "IntelOwlClient/3.1.2",
                 }
             )
             self._session = session
@@ -71,7 +73,6 @@ class IntelOwl:
         Returns:
             Dict: JSON body
         """
-        answer = None
         try:
             params = {"md5": md5, "analyzers_needed": analyzers_needed}
             if run_all_available_analyzers:
@@ -108,7 +109,8 @@ class IntelOwl:
         private_job: bool = False,
         disable_external_analyzers: bool = False,
         run_all_available_analyzers: bool = False,
-        runtime_configuration: Dict = {},
+        runtime_configuration: Dict = None,
+        tags: List[int] = None,
     ) -> Dict:
         """Send analysis request for a file.\n
         Endpoint: ``/api/send_analysis_request``
@@ -126,6 +128,8 @@ class IntelOwl:
                 Limit view permissions to your groups . Defaults to ``False``.
             disable_external_analyzers (bool, optional):
                 Disable analyzers that use external services. Defaults to ``False``.
+            tags (List[int]):
+                List of tags associated with this job
             run_all_available_analyzers (bool, optional):
                 If True, runs all compatible analyzers. Defaults to ``False``.
             runtime_configuration (Dict, optional):
@@ -137,12 +141,16 @@ class IntelOwl:
         Returns:
             Dict: JSON body
         """
-        answer = None
         try:
+            if not tags:
+                tags = []
+            if not runtime_configuration:
+                runtime_configuration = {}
             data = {
                 "is_sample": True,
                 "md5": self.get_md5(binary, type_="binary"),
                 "analyzers_requested": analyzers_requested,
+                "tags_id": tags,
                 "run_all_available_analyzers": run_all_available_analyzers,
                 "force_privacy": force_privacy,
                 "private": private_job,
@@ -165,7 +173,8 @@ class IntelOwl:
         private_job: bool = False,
         disable_external_analyzers: bool = False,
         run_all_available_analyzers: bool = False,
-        runtime_configuration: Dict = {},
+        runtime_configuration: Dict = None,
+        tags: List[int] = None,
     ) -> Dict:
         """Send analysis request for an observable.\n
         Endpoint: ``/api/send_analysis_request``
@@ -181,6 +190,8 @@ class IntelOwl:
                 Limit view permissions to your groups . Defaults to ``False``.
             disable_external_analyzers (bool, optional):
                 Disable analyzers that use external services. Defaults to ``False``.
+            tags (List[int]):
+                List of tags associated with this job
             run_all_available_analyzers (bool, optional):
                 If True, runs all compatible analyzers. Defaults to ``False``.
             runtime_configuration (Dict, optional):
@@ -192,12 +203,16 @@ class IntelOwl:
         Returns:
             Dict: JSON body
         """
-        answer = None
         try:
+            if not tags:
+                tags = []
+            if not runtime_configuration:
+                runtime_configuration = {}
             data = {
                 "is_sample": False,
                 "md5": self.get_md5(observable_name, type_="observable"),
                 "analyzers_requested": analyzers_requested,
+                "tags_id": tags,
                 "run_all_available_analyzers": run_all_available_analyzers,
                 "force_privacy": force_privacy,
                 "private": private_job,
@@ -268,33 +283,64 @@ class IntelOwl:
         )
         answer = response.json()
         if answer.get("error", "") == "814":
-            err = """
-            Request failed..
-            Error: [i yellow]After the filter, no analyzers can be run.
-                Try with other analyzers.[/]
-            """
+            if self.cli:
+                err = """
+                    Request failed..
+                    Error: [i yellow]After the filter, no analyzers can be run.
+                        Try with other analyzers.[/]
+                    """
+            else:
+                err = "Request failed. After the filter, no analyzers can be run"
             raise IntelOwlClientException(err)
         warnings = answer.get("warnings", [])
-        self.logger.info(
-            f"""New Job running..
-            ID: {answer['job_id']} | Status: [u blue]{answer['status']}[/].
-            Got {len(warnings)} warnings:
-            [i yellow]{warnings if warnings else None}[/]
-        """
-        )
+        if self.cli:
+            info_log = f"""New Job running..
+                ID: {answer['job_id']} | Status: [u blue]{answer['status']}[/].
+                Got {len(warnings)} warnings:
+                [i yellow]{warnings if warnings else None}[/]
+            """
+        else:
+            info_log = f"""New Job running..
+                ID: {answer['job_id']} | Status: {answer['status']}.
+                Got {len(warnings)} warnings:
+                {warnings if warnings else None}
+            """
+        self.logger.info(info_log)
         response.raise_for_status()
         return answer
 
-    def ask_analysis_result(self, job_id):
+    def create_tag(self, label: str, color: str):
+        """Creates new tag by sending a POST Request
+        Endpoint: ``/api/tags``
+
+        Args:
+            label ([str]): [Label of the tag to be created]
+            color ([str]): [Color of the tag to be created]
         """
-        will be deprecated soon. Use `get_job_by_id` function instead.\n
-        Endpoint: ``/api/ask_analysis_result``
-        """
-        answer = None
         try:
-            params = {"job_id": job_id}
-            url = self.instance + "/api/ask_analysis_result"
-            response = self.session.get(url, params=params)
+            url = self.instance + "/api/tags"
+            data = {"label": label, "color": color}
+            response = self.session.post(url, data=data)
+            self.logger.debug(msg=(response.url, response.status_code))
+            response.raise_for_status()
+            answer = response.json()
+        except Exception as e:
+            raise IntelOwlClientException(e)
+        return answer
+
+    def edit_tag(self, tag_id: Union[int, str], label: str, color: str):
+        """Edits existing tag by sending PUT request
+        Endpoint: ``api/tags``
+
+        Args:
+            id ([int]): [Id of the existing tag]
+            label ([str]): [Label of the tag to be created]
+            color ([str]): [Color of the tag to be created]
+        """
+        try:
+            url = self.instance + "/api/tags/" + str(tag_id)
+            data = {"label": label, "color": color}
+            response = self.session.put(url, data=data)
             self.logger.debug(response.url)
             response.raise_for_status()
             answer = response.json()
@@ -307,7 +353,6 @@ class IntelOwl:
         Get current state of `analyzer_config.json` from the IntelOwl instance.\n
         Endpoint: ``/api/get_analyzer_configs``
         """
-        answer = None
         try:
             url = self.instance + "/api/get_analyzer_configs"
             response = self.session.get(url)
@@ -351,7 +396,6 @@ class IntelOwl:
         Returns:
             List[Dict[str, Any]]: List of jobs
         """
-        answer = None
         try:
             url = self.instance + "/api/jobs"
             response = self.session.get(url)
@@ -375,7 +419,6 @@ class IntelOwl:
         Returns:
             Dict[str, str]: Dict with 3 keys: `id`, `label` and `color`.
         """
-        answer = None
         try:
             url = self.instance + "/api/tags/"
             response = self.session.get(url + str(tag_id))
@@ -399,7 +442,6 @@ class IntelOwl:
         Returns:
             Dict[str, Any]: JSON body.
         """
-        answer = None
         try:
             url = self.instance + "/api/jobs/" + str(job_id)
             response = self.session.get(url)
@@ -413,7 +455,7 @@ class IntelOwl:
     @staticmethod
     def get_md5(
         to_hash: AnyStr,
-        type_: Union["observable", "binary", "file"] = "observable",
+        type_="observable",
     ) -> str:
         """Returns md5sum of given observable or file object.
 
@@ -447,27 +489,30 @@ class IntelOwl:
         obj: str,
         type_: str,
         analyzers_list: List[str],
+        tags_list: List[int],
         run_all: bool,
         force_privacy,
         private_job,
         disable_external_analyzers,
         check,
-        runtime_configuration: Dict = {},
+        runtime_configuration: Dict = None,
         should_poll: bool = False,
     ) -> None:
         """
         For internal use by the pyintelowl CLI.
         """
+        if not runtime_configuration:
+            runtime_configuration = {}
         # CLI sanity checks
         if analyzers_list and run_all:
-            self.logger.warn(
+            self.logger.warning(
                 """
                 Can't use -al and -aa options together. See usage with -h.
                 """
             )
             return
         if not (analyzers_list or run_all):
-            self.logger.warn(
+            self.logger.warning(
                 """
                 Either one of -al, -aa must be specified. See usage with -h.
                 """,
@@ -505,6 +550,7 @@ class IntelOwl:
                 force_privacy=force_privacy,
                 private_job=private_job,
                 disable_external_analyzers=disable_external_analyzers,
+                tags=tags_list,
                 run_all_available_analyzers=run_all,
                 runtime_configuration=runtime_configuration,
             )
@@ -517,6 +563,7 @@ class IntelOwl:
                 force_privacy=force_privacy,
                 private_job=private_job,
                 disable_external_analyzers=disable_external_analyzers,
+                tags=tags_list,
                 run_all_available_analyzers=run_all,
                 runtime_configuration=runtime_configuration,
             )
@@ -536,8 +583,7 @@ class IntelOwl:
                 """
             )
 
-    @staticmethod
-    def _get_observable_classification(value: str) -> str:
+    def _get_observable_classification(self, value: str) -> str:
         """Returns observable classification for the given value.\n
         Only following types are supported:
         ip, domain, url, hash (md5, sha1, sha256)
@@ -571,7 +617,8 @@ class IntelOwl:
             ):
                 classification = "hash"
             else:
-                raise IntelOwlClientException(
+                classification = "general"
+                self.logger.warning(
                     f"{value} is neither a domain nor a URL nor a IP not a hash"
                 )
         else:
@@ -579,3 +626,84 @@ class IntelOwl:
             classification = "ip"
 
         return classification
+
+    def kill_running_job(self, job_id: int) -> bool:
+        """Send kill_running_job request.\n
+        Method: PATCH
+        Endpoint: ``/api/jobs/{job_id}/kill``
+
+        Args:
+            job_id (int):
+                id of job to kill
+
+        Raises:
+            IntelOwlClientException: on client/HTTP error
+
+        Returns:
+            Bool: killed or not
+        """
+
+        killed = False
+        try:
+            url = self.instance + f"/api/jobs/{job_id}/kill"
+            response = self.session.patch(url)
+            self.logger.debug(msg=(response.url, response.status_code))
+            killed = response.status_code == 200
+            response.raise_for_status()
+        except Exception as e:
+            raise IntelOwlClientException(e)
+        return killed
+
+    def delete_job_by_id(self, job_id: int) -> bool:
+        """Send delete job request.\n
+        Method: DELETE
+        Endpoint: ``/api/jobs/{job_id}``
+
+        Args:
+            job_id (int):
+                id of job to kill
+
+        Raises:
+            IntelOwlClientException: on client/HTTP error
+
+        Returns:
+            Bool: deleted or not
+        """
+
+        deleted = False
+        try:
+            url = self.instance + "/api/jobs/" + str(job_id)
+            response = self.session.delete(url)
+            self.logger.debug(msg=(response.url, response.status_code))
+            deleted = response.status_code == 204
+            response.raise_for_status()
+        except Exception as e:
+            raise IntelOwlClientException(e)
+        return deleted
+
+    def delete_tag_by_id(self, tag_id: int) -> bool:
+        """Send delete tag request.\n
+        Method: DELETE
+        Endpoint: ``/api/tags/{tag_id}``
+
+        Args:
+            tag_id (int):
+                id of tag to delete
+
+        Raises:
+            IntelOwlClientException: on client/HTTP error
+
+        Returns:
+            Bool: deleted or not
+        """
+
+        deleted = False
+        try:
+            url = self.instance + "/api/tags/" + str(tag_id)
+            response = self.session.delete(url)
+            self.logger.debug(msg=(response.url, response.status_code))
+            deleted = response.status_code == 204
+            response.raise_for_status()
+        except Exception as e:
+            raise IntelOwlClientException(e)
+        return deleted
